import {
  boolean,
  index,
  integer,
  pgEnum,
  pgTableCreator,
  text,
  timestamp,
  uuid,
  unique,
} from "drizzle-orm/pg-core";
import { relations } from "drizzle-orm";

export const createTable = pgTableCreator((name) => `wine-shop_${name}`);

// Producer Schema
export const producers = createTable(
  "producer",
  {
    id: uuid("id").primaryKey().defaultRandom(),
    name: text("name").notNull().unique(),
    description: text("description"),
    imageUrl: text("imageUrl"),
    countryId: uuid("countryId").notNull(),
    createdAt: timestamp("createdAt").defaultNow().notNull(),
    updatedAt: timestamp("updatedAt").defaultNow(),
  },
  (producers) => {
    return {
      nameIdx: index("producer_name").on(producers.name),
    };
  },
);

// one-to-many relation producer -> wines
export const producersRelations = relations(producers, ({ many }) => ({
  wines: many(wines),
}));

// Wines schema
export const typeEnum = pgEnum("type", [
  "sparkling",
  "white",
  "red",
  "sweet",
  "other",
]);

export const wines = createTable("wine", {
  id: uuid("id").primaryKey().defaultRandom(),
  name: text("name").notNull(),
  type: typeEnum("type").notNull(),
  description: text("description"),
  imageUrl: text("imageUrl"),
  producerId: uuid("producerId").notNull(),
  subRegionId: uuid("subRegionId"),
  regionId: uuid("regionId").notNull(),
  countryId: uuid("countryId").notNull(),
  price: integer("price").notNull(),
  inStock: boolean("inStock").notNull().default(false),
  createdAt: timestamp("createdAt").defaultNow().notNull(),
  updatedAt: timestamp("updatedAt").defaultNow(),
});

// many-to-one relationship wine -> producer
export const winesRelations = relations(wines, ({ one }) => ({
  producer: one(producers, {
    fields: [wines.producerId],
    references: [producers.id],
  }),
  subregion: one(subRegions, {
    fields: [wines.subRegionId],
    references: [subRegions.id],
  }),
  region: one(regions, {
    fields: [wines.regionId],
    references: [regions.id],
  }),
  country: one(countries, {
    fields: [wines.countryId],
    references: [countries.id],
  }),
}));

export const subRegions = createTable("subRegion", {
  id: uuid("id").primaryKey().notNull().defaultRandom(),
  name: text("name").notNull().unique(),
  regionId: uuid("regionId").notNull(),
});

export const subRegionsRelations = relations(subRegions, ({ many, one }) => ({
  wines: many(wines),
  region: one(regions, {
    fields: [subRegions.regionId],
    references: [regions.id],
  }),
}));

export const regions = createTable(
  "region",
  {
    id: uuid("id").primaryKey().notNull().defaultRandom(),
    name: text("name").notNull(),
    countryId: uuid("countryId").notNull(),
  },
  (t) => ({
    uniqueRegion: unique().on(t.name, t.countryId),
  }),
);

export const regionsRelations = relations(regions, ({ many, one }) => ({
  wines: many(wines),
  subRegions: many(subRegions),
  country: one(countries, {
    fields: [regions.countryId],
    references: [countries.id],
  }),
}));

export const countries = createTable("country", {
  id: uuid("id").primaryKey().notNull().defaultRandom(),
  name: text("name").notNull().unique(),
});

export const countriesRelations = relations(countries, ({ many }) => ({
  wines: many(wines),
  regions: many(regions),
}));
